@@ -12,72 +12,53 @@
  */
 
 #include "accountstate.h"
-#include "quotainfo.h"
+#include "accountmanager.h"
 #include "account.h"
 #include "creds/abstractcredentials.h"
+#include "logger.h"
+#include "configfile.h"
 
 #include <QDebug>
+#include <QSettings>
+#include <qfontmetrics.h>
 
 namespace OCC {
 
-Q_GLOBAL_STATIC(AccountStateManager, g_accountStateManager)
-
-AccountStateManager *AccountStateManager::instance()
-{
-    return g_accountStateManager();
-}
-
-AccountStateManager::AccountStateManager()
-    : _accountState(0)
-{
-    connect(AccountManager::instance(), SIGNAL(accountAdded(AccountPtr)),
-            SLOT(slotAccountAdded(AccountPtr)));
-}
-
-AccountStateManager::~AccountStateManager()
-{}
-
-void AccountStateManager::setAccountState(AccountState *accountState)
-{
-    if (_accountState) {
-        emit accountStateRemoved(_accountState);
-    }
-    _accountState = accountState;
-    if (accountState) {
-        emit accountStateAdded(accountState);
-    }
-}
-
-void AccountStateManager::slotAccountAdded(AccountPtr account)
-{
-    setAccountState(new AccountState(account));
-}
-
 AccountState::AccountState(AccountPtr account)
-    : QObject(account.data())
+    : QObject()
     , _account(account)
-    , _quotaInfo(0)
     , _state(AccountState::Disconnected)
     , _connectionStatus(ConnectionValidator::Undefined)
     , _waitingForNewCredentials(false)
 {
     qRegisterMetaType<AccountState*>("AccountState*");
 
-    _quotaInfo = new QuotaInfo(this); // Need to be initialized when 'this' is fully initialized
-
     connect(account.data(), SIGNAL(invalidCredentials()),
             SLOT(slotInvalidCredentials()));
     connect(account.data(), SIGNAL(credentialsFetched(AbstractCredentials*)),
             SLOT(slotCredentialsFetched(AbstractCredentials*)));
+    connect(account.data(), SIGNAL(credentialsAsked(AbstractCredentials*)),
+            SLOT(slotCredentialsAsked(AbstractCredentials*)));
+    _timeSinceLastETagCheck.invalidate();
 }
 
 AccountState::~AccountState()
 {
 }
 
+AccountState *AccountState::loadFromSettings(AccountPtr account, QSettings& /*settings*/)
+{
+    auto accountState = new AccountState(account);
+    return accountState;
+}
+
+void AccountState::writeToSettings(QSettings& /*settings*/)
+{
+}
+
 AccountPtr AccountState::account() const
 {
-    return _account.toStrongRef();
+    return _account;
 }
 
 AccountState::ConnectionStatus AccountState::connectionStatus() const
@@ -114,6 +95,9 @@ void AccountState::setState(State state)
         } else if (oldState == SignedOut && _state == Disconnected) {
             checkConnectivity();
         }
+        if (oldState == Connected || _state == Connected) {
+            emit isConnectedChanged();
+        }
     }
 
     // might not have changed but the underlying _connectionErrors might have
@@ -125,19 +109,19 @@ QString AccountState::stateString(State state)
     switch (state)
     {
     case SignedOut:
-        return QLatin1String("SignedOut");
+        return tr("Signed out");
     case Disconnected:
-        return QLatin1String("Disconnected");
+        return tr("Disconnected");
     case Connected:
-        return QLatin1String("Connected");
+        return tr("Connected");
     case ServiceUnavailable:
-        return QLatin1String("ServiceUnavailable");
+        return tr("Service unavailable");
     case NetworkError:
-        return QLatin1String("NetworkError");
+        return tr("Network error");
     case ConfigurationError:
-        return QLatin1String("ConfigurationError");
+        return tr("Configuration error");
     }
-    return QLatin1String("Unknown");
+    return tr("Unknown account state");
 }
 
 bool AccountState::isSignedOut() const
@@ -145,11 +129,15 @@ bool AccountState::isSignedOut() const
     return _state == SignedOut;
 }
 
-void AccountState::setSignedOut(bool signedOut)
+void AccountState::signOutByUi()
 {
-    if (signedOut) {
-        setState(SignedOut);
-    } else {
+    account()->credentials()->forgetSensitiveData();
+    setState(SignedOut);
+}
+
+void AccountState::signIn()
+{
+    if (_state == SignedOut) {
         setState(Disconnected);
     }
 }
@@ -164,9 +152,9 @@ bool AccountState::isConnectedOrTemporarilyUnavailable() const
     return isConnected() || _state == ServiceUnavailable;
 }
 
-QuotaInfo *AccountState::quotaInfo()
+void AccountState::tagLastSuccessfullETagRequest()
 {
-    return _quotaInfo;
+    _timeSinceLastETagCheck.restart();
 }
 
 void AccountState::checkConnectivity()
@@ -176,9 +164,21 @@ void AccountState::checkConnectivity()
     }
 
     if (_connectionValidator) {
-        qDebug() << "ConnectionValidator already running, ignoring";
+        qDebug() << "ConnectionValidator already running, ignoring" << account()->displayName();
         return;
     }
+
+    // IF the account is connected the connection check can be skipped
+    // if the last successful etag check job is not so long ago.
+    ConfigFile cfg;
+    int polltime = cfg.remotePollInterval();
+
+    if (isConnected() && _timeSinceLastETagCheck.isValid()
+            && _timeSinceLastETagCheck.elapsed() < polltime) {
+        //qDebug() << account()->displayName() << "The last ETag check succeeded within the last " << polltime/1000 << " secs. No connection check needed!";
+        return;
+    }
+
     ConnectionValidator * conValidator = new ConnectionValidator(account());
     _connectionValidator = conValidator;
     connect(conValidator, SIGNAL(connectionResult(ConnectionValidator::Status,QStringList)),
@@ -197,6 +197,10 @@ void AccountState::checkConnectivity()
         // As an attempted workaround, reset the QNAM regularly if the account is
         // disconnected.
         account()->resetNetworkAccessManager();
+
+        // If we don't reset the ssl config a second CheckServerJob can produce a
+        // ssl config that does not have a sensible certificate chain.
+        account()->setSslConfiguration(QSslConfiguration());
 #endif
         conValidator->checkServerAndAuth();
     }
@@ -208,7 +212,6 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
         return;
     }
 
-    auto oldStatus = _connectionStatus;
     if (_connectionStatus != status) {
         qDebug() << "AccountState connection status change: "
                  << connectionStatusString(_connectionStatus) << "->"
@@ -220,7 +223,7 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
     switch (status)
     {
     case ConnectionValidator::Connected:
-        if (oldStatus != ConnectionValidator::Connected) {
+        if (_state != Connected) {
             setState(Connected);
         }
         break;
@@ -237,8 +240,8 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
         // much more likely, so keep trying to connect.
         setState(NetworkError);
         break;
-    case ConnectionValidator::CredentialsWrong:
-        account()->handleInvalidCredentials();
+    case ConnectionValidator::CredentialsMissingOrWrong:
+        slotInvalidCredentials();
         break;
     case ConnectionValidator::UserCanceledCredentials:
         setState(SignedOut);
@@ -254,15 +257,35 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
 
 void AccountState::slotInvalidCredentials()
 {
-    if (isSignedOut()) {
+    if (isSignedOut() || _waitingForNewCredentials)
         return;
-    }
+
+    if (account()->credentials()->ready())
+        account()->credentials()->invalidateToken();
+    account()->credentials()->fetchFromKeychain();
 
     setState(ConfigurationError);
     _waitingForNewCredentials = true;
 }
 
 void AccountState::slotCredentialsFetched(AbstractCredentials* credentials)
+{
+    if (!credentials->ready()) {
+        // No exiting credentials found in the keychain
+        credentials->askFromUser();
+        return;
+    }
+
+    _waitingForNewCredentials = false;
+
+    // When new credentials become available we always want to restart the
+    // connection validation, even if it's currently running.
+    delete _connectionValidator;
+
+    checkConnectivity();
+}
+
+void AccountState::slotCredentialsAsked(AbstractCredentials* credentials)
 {
     _waitingForNewCredentials = false;
 
@@ -274,11 +297,36 @@ void AccountState::slotCredentialsFetched(AbstractCredentials* credentials)
 
     // When new credentials become available we always want to restart the
     // connection validation, even if it's currently running.
-    if (_connectionValidator) {
-        delete _connectionValidator;
-    }
+    delete _connectionValidator;
 
     checkConnectivity();
 }
 
+std::unique_ptr<QSettings> AccountState::settings()
+{
+    auto s = _account->settingsWithGroup(QLatin1String("Accounts"));
+    s->beginGroup(_account->id());
+    return s;
+}
+
+QString AccountState::shortDisplayNameForSettings(int width) const
+{
+    QString user = account()->credentials()->user();
+    QString host = account()->url().host();
+    int port = account()->url().port();
+    if (port > 0 && port != 80 && port != 443) {
+        host.append(QLatin1Char(':'));
+        host.append(QString::number(port));
+    }
+    if (width > 0) {
+        QFont f;
+        QFontMetrics fm(f);
+        host = fm.elidedText(host, Qt::ElideMiddle, width);
+        user = fm.elidedText(user, Qt::ElideRight, width);
+    }
+    return user + QLatin1String("\n") + host;
+}
+
+
+
 } // namespace OCC
