@@ -19,7 +19,6 @@
 #include "utility.h"
 
 #include "creds/abstractcredentials.h"
-#include "creds/credentialsfactory.h"
 
 #ifndef TOKEN_AUTH_ONLY
 #include <QWidget>
@@ -43,13 +42,16 @@ namespace OCC {
 //static const char caCertsKeyC[] = "CaCertificates"; only used from account.cpp
 static const char remotePollIntervalC[] = "remotePollInterval";
 static const char forceSyncIntervalC[] = "forceSyncInterval";
+static const char notificationRefreshIntervalC[] = "notificationRefreshInterval";
 static const char monoIconsC[] = "monoIcons";
+static const char promptDeleteC[] = "promptDeleteAllFiles";
 static const char crashReporterC[] = "crashReporter";
 static const char optionalDesktopNoficationsC[] = "optionalDesktopNotifications";
 static const char skipUpdateCheckC[] = "skipUpdateCheck";
+static const char updateCheckIntervalC[] = "updateCheckInterval";
 static const char geometryC[] = "geometry";
 static const char timeoutC[] = "timeout";
-static const char transmissionChecksumC[] = "transmissionChecksum";
+static const char chunkSizeC[] = "chunkSize";
 
 static const char proxyHostC[] = "Proxy/host";
 static const char proxyTypeC[] = "Proxy/type";
@@ -63,6 +65,9 @@ static const char useDownloadLimitC[] = "BWLimit/useDownloadLimit";
 static const char uploadLimitC[]      = "BWLimit/uploadLimit";
 static const char downloadLimitC[]    = "BWLimit/downloadLimit";
 
+static const char newBigFolderSizeLimitC[] = "newBigFolderSizeLimit";
+static const char useNewBigFolderSizeLimitC[] = "useNewBigFolderSizeLimit";
+
 static const char maxLogLinesC[] = "Logging/maxLogLines";
 
 const char certPath[] = "http_certificatePath";
@@ -92,11 +97,9 @@ bool ConfigFile::setConfDir(const QString &value)
     if( dirPath.isEmpty() ) return false;
 
     QFileInfo fi(dirPath);
-    if ( !fi.exists() && !fi.isAbsolute() ) {
-        QDir::current().mkdir(dirPath);
-        QDir dir = QDir::current();
-        dir.cd(dirPath);
-        fi.setFile(dir.path());
+    if ( !fi.exists()  ) {
+        QDir().mkpath(dirPath);
+        fi.setFile(dirPath);
     }
     if( fi.exists() && fi.isDir() ) {
         dirPath = fi.absoluteFilePath();
@@ -119,18 +122,10 @@ int ConfigFile::timeout() const
     return settings.value(QLatin1String(timeoutC), 300).toInt(); // default to 5 min
 }
 
-QString ConfigFile::transmissionChecksum() const
+quint64 ConfigFile::chunkSize() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
-
-    QString checksum = settings.value(QLatin1String(transmissionChecksumC), QString()).toString();
-
-    if( checksum.isEmpty() ) {
-        // if the config file setting is empty, maybe the Branding requires it.
-        checksum = Theme::instance()->transmissionChecksum();
-    }
-
-    return checksum;
+    return settings.value(QLatin1String(chunkSizeC), 10*1000*1000).toLongLong(); // default to 10 MB
 }
 
 void ConfigFile::setOptionalDesktopNotifications(bool show)
@@ -354,7 +349,7 @@ void ConfigFile::setRemotePollInterval(int interval, const QString &connection )
     if( connection.isEmpty() ) con = defaultConnection();
 
     if( interval < 5000 ) {
-        qDebug() << "Remote Poll interval of " << interval << " is below fife seconds.";
+        qDebug() << "Remote Poll interval of " << interval << " is below five seconds.";
         return;
     }
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -381,6 +376,40 @@ quint64 ConfigFile::forceSyncInterval(const QString& connection) const
     return interval;
 }
 
+quint64 ConfigFile::notificationRefreshInterval(const QString& connection) const
+{
+    QString con( connection );
+    if( connection.isEmpty() ) con = defaultConnection();
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.beginGroup( con );
+
+    quint64 defaultInterval = 5 * 60 * 1000ull; // 5 minutes
+    quint64 interval = settings.value( QLatin1String(notificationRefreshIntervalC), defaultInterval ).toULongLong();
+    if( interval < 60*1000ull) {
+        qDebug() << "notification refresh interval smaller than one minute, setting to one minute";
+        interval = 60*1000ull;
+    }
+    return interval;
+}
+
+int ConfigFile::updateCheckInterval( const QString& connection ) const
+{
+    QString con( connection );
+    if( connection.isEmpty() ) con = defaultConnection();
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.beginGroup( con );
+
+    int defaultInterval = 1000*60*60*10; // ten hours
+    int interval = settings.value( QLatin1String(updateCheckIntervalC), defaultInterval ).toInt();
+
+    int minInterval = 1000*60*5;
+    if( interval < minInterval) {
+        qDebug() << "Update check interval less than five minutes, setting " << minInterval;
+        interval = minInterval;
+    }
+    return interval;
+}
+
 bool ConfigFile::skipUpdateCheck( const QString& connection ) const
 {
     QString con( connection );
@@ -481,6 +510,9 @@ void ConfigFile::setValue(const QString& key, const QVariant &value)
 
 int ConfigFile::proxyType() const
 {
+    if (Theme::instance()->forceSystemNetworkProxy()) {
+        return QNetworkProxy::DefaultProxy;
+    }
     return getValue(QLatin1String(proxyTypeC)).toInt();
 }
 
@@ -515,9 +547,9 @@ int ConfigFile::useUploadLimit() const
     return getValue(useUploadLimitC, QString::null, 0).toInt();
 }
 
-bool ConfigFile::useDownloadLimit() const
+int ConfigFile::useDownloadLimit() const
 {
-    return getValue(useDownloadLimitC, QString::null, false).toBool();
+    return getValue(useDownloadLimitC, QString::null, 0).toInt();
 }
 
 void ConfigFile::setUseUploadLimit(int val)
@@ -525,9 +557,9 @@ void ConfigFile::setUseUploadLimit(int val)
     setValue(useUploadLimitC, val);
 }
 
-void ConfigFile::setUseDownloadLimit(bool enable)
+void ConfigFile::setUseDownloadLimit(int val)
 {
-    setValue(useDownloadLimitC, enable);
+    setValue(useDownloadLimitC, val);
 }
 
 int ConfigFile::uploadLimit() const
@@ -550,6 +582,32 @@ void ConfigFile::setDownloadLimit(int kbytes)
     setValue(downloadLimitC, kbytes);
 }
 
+QPair<bool, quint64> ConfigFile::newBigFolderSizeLimit() const
+{
+    auto defaultValue = Theme::instance()->newBigFolderSizeLimit();
+    qint64 value = getValue(newBigFolderSizeLimitC, QString(), defaultValue).toLongLong();
+    bool use = value >= 0 && getValue(useNewBigFolderSizeLimitC, QString(), true).toBool();
+    return qMakePair(use, quint64(qMax<qint64>(0, value)));
+}
+
+void ConfigFile::setNewBigFolderSizeLimit(bool isChecked, quint64 mbytes)
+{
+    setValue(newBigFolderSizeLimitC, mbytes);
+    setValue(useNewBigFolderSizeLimitC, isChecked);
+}
+
+bool ConfigFile::promptDeleteFiles() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(promptDeleteC), true).toBool();
+}
+
+void ConfigFile::setPromptDeleteFiles(bool promptDeleteFiles)
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(promptDeleteC), promptDeleteFiles);
+}
+
 bool ConfigFile::monoIcons() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
